@@ -3,7 +3,7 @@
     Copyright (C) 2002-2014 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
 
-    Copyright (C) 2012-2014 by Khaled Hosny <khaledhosny@eglug.org>
+    Copyright (C) 2012-2015 by Khaled Hosny <khaledhosny@eglug.org>
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
 
@@ -61,8 +61,13 @@
 #include "dvipdfmx.h"
 
 #ifdef XETEX
+#include "dpxfile.h"
 #include "pdfximage.h"
-#include FT_ADVANCES_H
+#include "tt_aux.h"
+#include "tt_table.h"
+#include "t1_load.h"
+#include "t1_char.h"
+#include "cff_dict.h"
 #endif
 
 #define DVI_STACK_DEPTH_MAX  256u
@@ -142,7 +147,13 @@ static struct loaded_font
   int   source;     /* Source is either DVI or VF */
 #ifdef XETEX
   uint32_t rgba_color;
-  FT_Face ft_face;
+  struct tt_longMetrics *hvmt;
+  int   ascent;
+  int   descent;
+  unsigned unitsPerEm;
+  cff_font *cffont;
+  int cff_is_standard_encoding;
+  unsigned numGlyphs;
   int   layout_dir;
   float extend;
   float slant;
@@ -793,6 +804,8 @@ dvi_locate_font (const char *tfm_name, spt_t ptsize)
     subfont_id = sfd_load_record(mrec->charmap.sfd_name, mrec->charmap.subfont_id);
   }
 
+  memset(&loaded_fonts[cur_id], 0, sizeof (struct loaded_font));
+
   /* TFM must exist here. */
   loaded_fonts[cur_id].tfm_id     = tfm_open(tfm_name, 1);
   loaded_fonts[cur_id].subfont_id = subfont_id;
@@ -913,31 +926,116 @@ dvi_locate_native_font (const char *filename, uint32_t index,
 {
   int           cur_id = -1;
   fontmap_rec  *mrec;
-  char         *fontmap_key = malloc(strlen(filename) + 40); // CHECK this is enough
+  char         *fontmap_key;
+  FILE         *fp;
+  char         *path;
+  sfnt         *sfont;
+  unsigned      offset = 0;
+  struct tt_head_table *head;
+  struct tt_maxp_table *maxp;
+  struct tt_hhea_table *hhea;
+  int is_dfont = 0, is_type1 = 0;
 
   if (verbose)
     MESG("<%s@%.2fpt", filename, ptsize * dvi2pts);
 
+  if ((path = dpx_find_dfont_file(filename)) != NULL &&
+      (fp = fopen(path, "rb")) != NULL)
+    is_dfont = 1;
+  else if ((path = dpx_find_type1_file(filename)) != NULL)
+    is_type1 = 1;
+  else if (((path = dpx_find_opentype_file(filename)) == NULL
+         && (path = dpx_find_truetype_file(filename)) == NULL)
+         || (fp = fopen(path, "rb")) == NULL) {
+    ERROR("Cannot proceed without the font: %s", filename);
+  }
   need_more_fonts(1);
 
   cur_id = num_loaded_fonts++;
 
-  sprintf(fontmap_key, "%s/%u/%c/%d/%d/%d", filename, index, layout_dir == 0 ? 'H' : 'V', extend, slant, embolden);
+  fontmap_key = malloc(strlen(path) + 40); // CHECK this is enough
+  sprintf(fontmap_key, "%s/%u/%c/%d/%d/%d", path, index, layout_dir == 0 ? 'H' : 'V', extend, slant, embolden);
   mrec = pdf_lookup_fontmap_record(fontmap_key);
   if (mrec == NULL) {
-    if (pdf_load_native_font(filename, index, layout_dir, extend, slant, embolden) == -1) {
-      ERROR("Cannot proceed without the \"native\" font: %s", filename);
+    if ((mrec = pdf_insert_native_fontmap_record(path, index, layout_dir, extend, slant, embolden)) == NULL) {
+      ERROR("Failed to insert font record for font: %s", filename);
     }
-    mrec = pdf_lookup_fontmap_record(fontmap_key);
-    /* FIXME: would be more efficient if pdf_load_native_font returned the mrec ptr (or NULL for error)
-              so we could avoid doing a second lookup for the item we just inserted */
   }
+
+  memset(&loaded_fonts[cur_id], 0, sizeof (struct loaded_font));
+
   loaded_fonts[cur_id].font_id = pdf_dev_locate_font(fontmap_key, ptsize);
   loaded_fonts[cur_id].size    = ptsize;
   loaded_fonts[cur_id].type    = NATIVE;
   free(fontmap_key);
 
-  loaded_fonts[cur_id].ft_face = mrec->opt.ft_face;
+  if (is_type1) {
+    cff_font *cffont;
+    char     *enc_vec[256];
+    int       code;
+
+    fp = DPXFOPEN(filename, DPX_RES_TYPE_T1FONT);
+    if (!fp)
+      return -1;
+
+    if (!is_pfb(fp))
+      ERROR("Failed to read Type 1 font \"%s\".", filename);
+
+    memset(enc_vec, 0, 256 * sizeof(char *));
+    cffont = t1_load_font(enc_vec, 0, fp);
+    if (!cffont)
+      ERROR("Failed to read Type 1 font \"%s\".", filename);
+
+    loaded_fonts[cur_id].cffont = cffont;
+    loaded_fonts[cur_id].cff_is_standard_encoding = enc_vec[0] == NULL;
+
+    if (cff_dict_known(cffont->topdict, "FontBBox")) {
+      loaded_fonts[cur_id].ascent = cff_dict_get(cffont->topdict, "FontBBox", 3);
+      loaded_fonts[cur_id].descent = cff_dict_get(cffont->topdict, "FontBBox", 1);
+    } else {
+      loaded_fonts[cur_id].ascent = 690;
+      loaded_fonts[cur_id].descent = -190;
+    }
+
+    loaded_fonts[cur_id].unitsPerEm = 1000;
+    loaded_fonts[cur_id].numGlyphs = cffont->num_glyphs;
+
+    DPXFCLOSE(fp);
+  } else {
+    if (is_dfont)
+      sfont = dfont_open(fp, index);
+    else
+      sfont = sfnt_open(fp);
+    if (sfont->type == SFNT_TYPE_TTC)
+      offset = ttc_read_offset(sfont, index);
+    else if (sfont->type == SFNT_TYPE_DFONT)
+      offset = sfont->offset;
+    sfnt_read_table_directory(sfont, offset);
+    head = tt_read_head_table(sfont);
+    maxp = tt_read_maxp_table(sfont);
+    hhea = tt_read_hhea_table(sfont);
+    loaded_fonts[cur_id].ascent = hhea->ascent;
+    loaded_fonts[cur_id].descent = hhea->descent;
+    loaded_fonts[cur_id].unitsPerEm = head->unitsPerEm;
+    loaded_fonts[cur_id].numGlyphs = maxp->numGlyphs;
+    if (layout_dir == 1 && sfnt_find_table_pos(sfont, "vmtx") > 0) {
+      struct tt_vhea_table *vhea = tt_read_vhea_table(sfont);
+      sfnt_locate_table(sfont, "vmtx");
+      loaded_fonts[cur_id].hvmt = tt_read_longMetrics(sfont, maxp->numGlyphs, vhea->numOfLongVerMetrics, vhea->numOfExSideBearings);
+      RELEASE(vhea);
+    } else {
+      sfnt_locate_table(sfont, "hmtx");
+      loaded_fonts[cur_id].hvmt = tt_read_longMetrics(sfont, maxp->numGlyphs, hhea->numOfLongHorMetrics, hhea->numOfExSideBearings);
+    }
+    RELEASE(hhea);
+    RELEASE(maxp);
+    RELEASE(head);
+    sfnt_close(sfont);
+    fclose(fp);
+  }
+
+  free(path);
+
   loaded_fonts[cur_id].layout_dir = layout_dir;
   loaded_fonts[cur_id].extend = mrec->opt.extend;
   loaded_fonts[cur_id].slant = mrec->opt.slant;
@@ -1474,13 +1572,13 @@ dvi_end_reflect (void)
     current_font = lr_state.font;
     dvi_page_buf_index = lr_state.buf_index;
     lr_mode = REVERSE(lr_state.state); /* must precede dvi_right */
-    dvi_right(-lr_width);
+    dvi_right(-(int32_t)lr_width);
     lr_width_push();
     break;
   case LTYPESETTING:
   case RTYPESETTING:
     lr_width_pop();
-    dvi_right(-lr_width);
+    dvi_right(-(int32_t)lr_width);
     lr_mode = REVERSE(lr_mode);
     break;
   default:                     /* lr_mode > SKIMMING */
@@ -1566,24 +1664,37 @@ do_glyphs (void)
 
   for (i = 0; i < slen; i++) {
     glyph_id = get_buffered_unsigned_pair(); /* freetype glyph index */
-    if (glyph_id < font->ft_face->num_glyphs) {
-      FT_Error error;
-      FT_Fixed advance;
-      int flags = FT_LOAD_NO_SCALE;
+    if (glyph_id < font->numGlyphs) {
+      unsigned advance;
+      double ascent = (double)font->ascent;
+      double descent = (double)font->descent;
 
-      if (font->layout_dir == 1)
-        flags |= FT_LOAD_VERTICAL_LAYOUT;
+      if (font->cffont) {
+        cff_index *cstrings = font->cffont->cstrings;
+        t1_ginfo gm;
 
-      error = FT_Get_Advance(font->ft_face, glyph_id, flags, &advance);
-      if (error)
-        advance = 0;
+        /* For standard encoding, Type1 glyph id is FreeType glyph index + 1. */
+        if (font->cff_is_standard_encoding)
+          glyph_id += 1;
+
+        t1char_get_metrics(cstrings->data + cstrings->offset[glyph_id] - 1,
+                           cstrings->offset[glyph_id + 1] - cstrings->offset[glyph_id],
+                           font->cffont->subrs[0], &gm);
+
+        advance = font->layout_dir == 0 ? gm.wx : gm.wy;
+        ascent = gm.bbox.ury;
+        descent = gm.bbox.lly;
+      } else {
+        advance = font->hvmt[glyph_id].advance;
+      }
+
+      glyph_width    = (double)font->size * (double)advance / (double)font->unitsPerEm;
+      glyph_width    = glyph_width * font->extend;
 
-      glyph_width = (double)font->size * (double)advance / (double)font->ft_face->units_per_EM;
-      glyph_width = glyph_width * font->extend;
       if (dvi_is_tracking_boxes()) {
         pdf_rect rect;
-        height = (double)font->size * (double)font->ft_face->ascender / (double)font->ft_face->units_per_EM;
-        depth  = (double)font->size * -(double)font->ft_face->descender / (double)font->ft_face->units_per_EM;
+        height = (double)font->size * ascent / (double)font->unitsPerEm;
+        depth  = (double)font->size * -descent / (double)font->unitsPerEm;
         pdf_dev_set_rect(&rect, dvi_state.h + xloc[i], -dvi_state.v - yloc[i], glyph_width, height, depth);
         pdf_doc_expand_box(&rect);
       }
@@ -1868,6 +1979,21 @@ dvi_close (void)
   page_loc  = NULL;
   num_pages = 0;
 
+#ifdef XETEX
+  for (i = 0; i < num_loaded_fonts; i++)
+  {
+    if (loaded_fonts[i].hvmt != NULL)
+      RELEASE(loaded_fonts[i].hvmt);
+
+    loaded_fonts[i].hvmt = NULL;
+
+    if (loaded_fonts[i].cffont != NULL)
+      cff_close(loaded_fonts[i].cffont);
+
+    loaded_fonts[i].cffont = NULL;
+  }
+#endif
+
   if (loaded_fonts)
     RELEASE(loaded_fonts);
   loaded_fonts     = NULL;
