@@ -2,7 +2,7 @@
 ** Matrix.cpp                                                           **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#define _USE_MATH_DEFINES
 #include <config.h>
 #include <algorithm>
 #include <cmath>
@@ -60,8 +61,7 @@ double det (const Matrix &m, int row, int col) {
 
 
 static inline double deg2rad (double deg) {
-	const double PI = acos(-1.0);
-	return PI*deg/180.0;
+	return M_PI*deg/180.0;
 }
 
 
