@@ -1,40 +1,39 @@
 #!/usr/bin/env perl
-# $Id: updmap.pl 30150 2013-04-28 23:33:28Z karl $
+# $Id: updmap.pl 37076 2015-04-27 17:08:10Z karl $
 # updmap - maintain map files for outline fonts.
 # (Maintained in TeX Live:Master/texmf-dist/scripts/texlive.)
 # 
-# Copyright 2011, 2012, 2013 Norbert Preining
+# Copyright 2011-2015 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
 # History:
 # Original shell script (C) 2002 Thomas Esser
-# first perl variant (C) Fabrice Popineau
+# first perl variant by Fabrice Popineau
 # later adaptions by Reinhard Kotucha and Karl Berry
 # the original versions were licensed under the following agreement:
 # Anyone may freely use, modify, and/or distribute this file, without
-# limitation.
-#
-# TODO
-# - check all other invocations
-# - after TL2012? Maybe remove support for reading updmap-local.cfg
-#
+
+my $svnid = '$Id: updmap.pl 37076 2015-04-27 17:08:10Z karl $';
 
 my $TEXMFROOT;
-
 BEGIN {
-  $^W=1;
+  $^W = 1;
   $TEXMFROOT = `kpsewhich -var-value=TEXMFROOT`;
   if ($?) {
-    print STDERR "updmap: Cannot find TEXMFROOT, aborting!\n";
-    exit 1;
+    die "$0: kpsewhich -var-value=TEXMFROOT failed, aborting early.\n";
   }
   chomp($TEXMFROOT);
-  unshift (@INC, "$TEXMFROOT/tlpkg");
+  unshift(@INC, "$TEXMFROOT/tlpkg");
 }
 
-
-my $version = '$Id: updmap.pl 30150 2013-04-28 23:33:28Z karl $';
+my $lastchdate = '$Date: 2015-04-27 19:08:10 +0200 (Mon, 27 Apr 2015) $';
+$lastchdate =~ s/^\$Date:\s*//;
+$lastchdate =~ s/ \(.*$//;
+my $svnrev = '$Revision: 37076 $';
+$svnrev =~ s/^\$Revision:\s*//;
+$svnrev =~ s/\s*\$$//;
+my $version = "r$svnrev ($lastchdate)";
 
 use Getopt::Long qw(:config no_autoabbrev ignore_case_always);
 use strict;
@@ -46,31 +45,37 @@ use TeXLive::TLUtils qw(mkdirhier mktexupd win32 basename dirname
 
 (my $prg = basename($0)) =~ s/\.pl$//;
 
-# sudo sometimes does not reset the home dir of root, check on that
-# see more comments at the definition of the function itself
-# this function checks by itself whether it is running on windows or not
+# sudo sometimes does not reset the home dir of root;
+# see more comments at the definition of this function.
 reset_root_home();
 
 chomp(my $TEXMFDIST = `kpsewhich --var-value=TEXMFDIST`);
 chomp(my $TEXMFVAR = `kpsewhich -var-value=TEXMFVAR`);
+chomp(my $TEXMFSYSVAR = `kpsewhich -var-value=TEXMFSYSVAR`);
 chomp(my $TEXMFCONFIG = `kpsewhich -var-value=TEXMFCONFIG`);
+chomp(my $TEXMFSYSCONFIG = `kpsewhich -var-value=TEXMFSYSCONFIG`);
 chomp(my $TEXMFHOME = `kpsewhich -var-value=TEXMFHOME`);
 
 # make sure that on windows *everything* is in lower case for comparison
 if (win32()) {
   $TEXMFDIST = lc($TEXMFDIST);
   $TEXMFVAR = lc($TEXMFVAR);
+  $TEXMFSYSVAR = lc($TEXMFSYSVAR);
   $TEXMFCONFIG = lc($TEXMFCONFIG);
+  $TEXMFSYSCONFIG = lc($TEXMFSYSCONFIG);
   $TEXMFROOT = lc($TEXMFROOT);
   $TEXMFHOME = lc($TEXMFHOME);
 }
 
+my $texmfconfig = $TEXMFCONFIG;
+my $texmfvar    = $TEXMFVAR;
 
 my %opts = ( quiet => 0, nohash => 0, nomkmap => 0 );
 my $alldata;
 my $updLSR;
 
 my @cmdline_options = (
+  "sys",
   "listfiles",
   "cnffile=s@", 
   "copy", 
@@ -145,8 +150,8 @@ my %settings = (
 
 &main();
 
-###############
-
+##################################################################
+#
 sub main {
   processOptions();
 
@@ -157,14 +162,39 @@ sub main {
     exit (0);
   }
 
-  if ($opts{'dvipdfmoutputdir'} && !defined($opts{'dvipdfmxoutputdir'})) {
-    $opts{'dvipdfmxoutputdir'} = $opts{'dvipdfmoutputdir'};
-    printf STDERR "Using --dvipdfmoutputdir options for dvipdfmx, but please use --dvipdfmxoutputdir\n";
+  # check if we are in *hidden* sys mode, in which case we switch
+  # to sys mode
+  # Nowdays we use -sys switch instead of simply overriding TEXMFVAR
+  # and TEXMFCONFIG
+  # This is used to warn users when they run updmap in usermode the first time.
+  # But it might happen that this script is called via another wrapper that
+  # sets TEXMFCONFIG and TEXMFVAR, and does not pass on the -sys option.
+  # for this case we check whether the SYS and non-SYS variants agree,
+  # and if, then switch to sys mode (with a warning)
+  if (($TEXMFSYSCONFIG eq $TEXMFCONFIG) && ($TEXMFSYSVAR eq $TEXMFVAR)) {
+    if (!$opts{'sys'}) {
+      print_warning("hidden sys mode found, switching to sys mode.\n");
+      $opts{'sys'} = 1;
+    }
   }
 
-  if ($opts{'dvipdfmoutputdir'} && $opts{'dvipdfmxoutputdir'} &&
-      $opts{'dvipdfmoutputdir'} ne $opts{'dvipdfmxoutputdir'}) {
-    printf STDERR "Options for --dvipdfmoutputdir and --dvipdfmxoutputdir do not agree\nplease use only --dvipdfmxoutputdir. Exiting.\n";
+  if ($opts{'sys'}) {
+    # we are running as updmap-sys, make sure that the right tree is used
+    $texmfconfig = $TEXMFSYSCONFIG;
+    $texmfvar    = $TEXMFSYSVAR;
+  }
+
+  if ($opts{'dvipdfmoutputdir'} && !defined($opts{'dvipdfmxoutputdir'})) {
+    $opts{'dvipdfmxoutputdir'} = $opts{'dvipdfmoutputdir'};
+    print_warning("Using --dvipdfmoutputdir options for dvipdfmx,"
+                  . " but please use --dvipdfmxoutputdir\n");
+  }
+
+  if ($opts{'dvipdfmoutputdir'} && $opts{'dvipdfmxoutputdir'}
+      && $opts{'dvipdfmoutputdir'} ne $opts{'dvipdfmxoutputdir'}) {
+    print_error("Options for --dvipdfmoutputdir and --dvipdfmxoutputdir"
+                . " do not match\n"
+                . "Please use only --dvipdfmxoutputdir; exiting.\n");
     exit(1);
   }
 
@@ -188,10 +218,10 @@ sub main {
         } elsif ($settings{$o}{'type'} eq "any") {
           print "(any string)\n";
         } else {
-          print "strange: unknown type of option $o\nplease report\n";
+          print_warning("strange: unknown type of option $o\nplease report\n");
         }
       } else {
-        print "$prg: unknown option: $o\n";
+        print_warning("unknown option: $o\n");
       }
     }
     exit 0;
@@ -235,26 +265,22 @@ sub main {
     }
     #
     # search for TEXMFLOCAL/web2c/updmap.cfg
-    # check for compatibility with old updmap-local.cfg
     my @tmlused;
     for my $tml (@TEXMFLOCAL) {
       my $TMLabs = Cwd::abs_path($tml);
       next if (!$TMLabs);
-      my $oldfound = 0;
-      if (-r "$TMLabs/web2c/updmap-local.cfg") {
-        push @tmlused, "$TMLabs/web2c/updmap-local.cfg";
-        warning("Old configuration file\n  $TMLabs/web2c/updmap-local.cfg\nfound! ");
-        $oldfound = 1;
-      }
       if (-r "$TMLabs/web2c/updmap.cfg") {
-        if ($oldfound) {
-          warning("Will read it *instead* of\n  $TMLabs/web2c/updmap.cfg\n");
-        } else {
-          push @tmlused, "$TMLabs/web2c/updmap.cfg";
-        }
+        push @tmlused, "$TMLabs/web2c/updmap.cfg";
+      }
+      #
+      # at least check for old updmap-local.cfg and warn!
+      if (-r "$TMLabs/web2c/updmap-local.cfg") {
+        print_warning("=============================\n");
+        print_warning("Old configuration file\n  $TMLabs/web2c/updmap-local.cfg\n");
+        print_warning("found! This file is *not* evaluated anymore, please move the information\n");
+        print_warning("to the file $TMLabs/updmap.cfg!\n");
+        print_warning("=============================\n");
       }
-      warning("Please consider moving the information from updmap-local.cfg to\n  $TMLabs/web2c/updmap.cfg\n")
-        if ($oldfound);
     }
     #
     # updmap (user):
@@ -274,16 +300,16 @@ sub main {
     # TEXMFLOCAL     $TEXLIVE/texmf-local/web2c/updmap.cfg
     # TEXMFDIST      $TEXLIVE/YYYY/texmf-dist/web2c/updmap.cfg
     #
-    @{$opts{'cnffile'}}  = @used_files;
+    @{$opts{'cnffile'}} = @used_files;
     #
-    # determine the config file that we will use for changes
-    # if in the list of used files contains either one from
+    # Determine the config file that we will use for changes:
+    # if the list of used files contains one from either
     # TEXMFHOME or TEXMFCONFIG (which is TEXMFSYSCONFIG in the -sys case)
-    # then use the *top* file (which will be either one of the two),
-    # if none of the two exists, create a file in TEXMFCONFIG and use it
+    # then use the *top* file (which will be one of the two *CONFIG);
+    # if neither of those two exists, create a file in TEXMFCONFIG and use it.
     my $use_top = 0;
     for my $f (@used_files) {
-      if ($f =~ m!(\Q$TEXMFHOME\E|\Q$TEXMFCONFIG\E)/web2c/updmap.cfg!) {
+      if ($f =~ m!(\Q$TEXMFHOME\E|\Q$texmfconfig\E)/web2c/updmap.cfg!) {
         $use_top = 1;
         last;
       }
@@ -292,16 +318,16 @@ sub main {
       ($changes_config_file) = @used_files;
     } else {
       # add the empty config file
-      my $dn = "$TEXMFCONFIG/web2c";
+      my $dn = "$texmfconfig/web2c";
       $changes_config_file = "$dn/updmap.cfg";
     }
   }
   if (!$opts{'quiet'}) {
-    print "$prg is using the following updmap.cfg files (in precedence order):\n";
+    print "$prg will read the following updmap.cfg files (in precedence order):\n";
     for my $f (@{$opts{'cnffile'}}) {
       print "  $f\n";
     }
-    print "$prg is using the following updmap.cfg file for writing changes:\n";
+    print "$prg may write changes to the following updmap.cfg file:\n";
     print "  $changes_config_file\n";
   }
   if ($opts{'listfiles'}) {
@@ -321,7 +347,7 @@ sub main {
         $v = "\"$v\"" if ($v =~ m/\s/);
         print "$o=$v ($vo)\n";
       } else {
-        printf STDERR "$prg: unknown option: $o\n";
+        print_warning("unknown option: $o\n");
       }
     }
     exit 0;
@@ -366,9 +392,8 @@ sub main {
         exit 0;
       }
       $changed ||= enable_disable_maps(@missing);
-      print "finished.\n";
-      # the original script did not run any update of the map files here,
-      # should we do that?
+      print "$0 --syncwithtrees finished.\n";
+      print "Now you need to run $prg normally to recreate map files.\n"
     }
     exit 0;
   }
@@ -376,7 +401,7 @@ sub main {
   my $cmd;
   if ($opts{'edit'}) {
     if ($opts{"dry-run"}) {
-      printf STDERR "No, are you joking, you want to edit with --dry-run?\n";
+      print_error("No, are you joking, you want to edit with --dry-run?\n");
       exit 1;
     }
     # it's not a good idea to edit updmap.cfg manually these days,
@@ -418,18 +443,18 @@ sub main {
       setupOutputDir("dvips");
       setupOutputDir("pdftex");
       setupOutputDir("dvipdfmx");
-      setupOutputDir("pxdvi");
+      # do pxdvi below, in mkmaps.
       merge_settings_replace_kanji();
       my @missing = read_map_files();
       if (@missing) {
-        print STDERR "\nERROR:  The following map file(s) couldn't be found:\n"; 
+        print_error("The following map file(s) couldn't be found:\n"); 
         for my $m (@missing) {
           my $orig = $alldata->{'maps'}{$m}{'origin'};
-          print STDERR "\t$m (in $orig)\n";
+          print_error("\t$m (in $orig)\n");
         }
-        print STDERR "\n\tDid you run mktexlsr?\n\n" .
+        print_error("Did you run mktexlsr?\n\n" .
           "\tYou can disable non-existent map entries using the option\n".
-          "\t  --syncwithtrees.\n\n";
+          "\t  --syncwithtrees.\n\n");
         exit 1;
       }
       merge_data();
@@ -451,8 +476,6 @@ sub main {
 
 ##################################################################
 #
-#
-
 sub getFonts {
   my ($first, @rest) = @_;
   my $getall = 0;
@@ -474,7 +497,7 @@ sub getFonts {
           if (defined($alldata->{'maps'}{$m}{'fonts'}{$k})) {
             push @lines, "$k " . $alldata->{'maps'}{$m}{'fonts'}{$k};
           } else {
-            print "undefined fonts for $k in $m   ?!?!?\n";
+            print_warning("undefined fonts for $k in $m   ?!?!?\n");
           }
           print LOG "$k\n" unless $opts{'dry-run'};
         }
@@ -770,8 +793,6 @@ sub transLW35 {
 #       ... %!PS fontname %other comments
 #       ... %!PS fontname
 #
-###############################################################################
-
 # reimplementation of the cryptic code that was there before
 sub cidx2dvips {
   my ($s) = @_;
@@ -840,8 +861,8 @@ sub cidx2dvips {
     # now we have the following format
     #  <word> <word> <word> some options like -e or -s
     if ($_ !~ m/([^ ][^ ]*) ([^ ][^ ]*) ([^ ][^ ]*)( (.*))?$/) {
-      print STDERR "cidx2dvips warning: Cannot translate font line:\n==> $l\n";
-      print STDERR "Current translation status: ==>$_==\n";
+      print_warning("cidx2dvips warning: Cannot translate font line:\n==> $l\n");
+      print_warning("Current translation status: ==>$_==\n");
       next;
     }
     my $tfmname = $1;
@@ -863,7 +884,7 @@ sub cidx2dvips {
       if ($italicmax > 0) {
         # we have already a definition of SlantFont via ,Italic or ,BoldItalic
         # warn the user that larger one is kept
-        print STDERR "cidx2dvips warning: Double slant specified via Italic and -s:\n==> $l\n==> Using only the biggest slant value.\n";
+        print_warning("cidx2dvips warning: Double slant specified via Italic and -s:\n==> $l\n==> Using only the biggest slant value.\n");
       }
       $italicmax = $1 if ($1 > $italicmax);
       $opts =~ s/-s ([.0-9-][.0-9-]*)//;
@@ -925,10 +946,10 @@ sub get_cfg {
 sub mkMaps {
   my $logfile;
 
-  $logfile = "$TEXMFVAR/web2c/updmap.log";
+  $logfile = "$texmfvar/web2c/updmap.log";
 
   if (! $opts{'dry-run'}) {
-    mkdirhier("$TEXMFVAR/web2c");
+    mkdirhier("$texmfvar/web2c");
     open LOG, ">$logfile"
         or die "$prg: Can't open log file \"$logfile\": $!";
     print LOG &version();
@@ -958,6 +979,10 @@ sub mkMaps {
   my ($kanjiEmbed, $kanjiEmbed_origin) = get_cfg('kanjiEmbed');
   my ($kanjiVariant, $kanjiVariant_origin) = get_cfg('kanjiVariant');
 
+  # pxdvi is optional, and off by default.  Don't create the output
+  # directory unless we are going to put something there.
+  setupOutputDir("pxdvi") if $pxdviUse eq "true";
+
   print_and_log ("\n$prg is creating new map files"
          . "\nusing the following configuration:"
          . "\n  LW35 font names                  : "
@@ -1045,6 +1070,7 @@ sub mkMaps {
   print_and_log (sprintf("         [%3d files]\n\n", scalar @notmixedmaps));
   only_log("\n");
 
+  my $first_time_creation_in_usermode = 0;
   # Create psfonts_t1.map, psfonts_pk.map, ps2pk.map and pdftex.map:
   my $dvipsoutputdir = $opts{'dvipsoutputdir'};
   my $pdftexoutputdir = $opts{'pdftexoutputdir'};
@@ -1062,6 +1088,9 @@ sub mkMaps {
     push @managed_files, "$pxdvioutputdir/xdvi-ptex.map"
       if ($pxdviUse eq "true");
     for my $file (@managed_files) {
+      if (!$opts{'sys'} && ! -r $file) {
+        $first_time_creation_in_usermode = 1;
+      }
       open FILE, ">$file";
       print FILE "% $file:\
 % maintained by updmap[-sys] (multi).\
@@ -1087,7 +1116,7 @@ sub mkMaps {
 
   if ($pxdviUse eq "true") {
     # we use the very same data as for kanjix.map, but generate
-    # a different file, so that in case a user wants to hand-craft it
+    # a different file, in case a user wants to hand-craft it
     print "Generating output for pxdvi...\n" if !$opts{'quiet'};
      &writeLines(">$pxdvioutputdir/xdvi-ptex.map", @kanjimaps_fonts);
   }
@@ -1194,31 +1223,55 @@ sub mkMaps {
         print_and_log ("\n");
       }
     } else {
-      print STDERR "$prg: Warning: File $d/$f doesn't exist.\n";
-      print LOG    "Warning: File $d/$f doesn't exist.\n" 
+      print_warning("File $d/$f doesn't exist.\n");
+      print LOG     "Warning: File $d/$f doesn't exist.\n" 
         unless $opts{'dry-run'};
     }
   }
 
+  sub check_mismatch {
+    my ($mm, $d, $f, $prog) = @_;
+    chomp (my $kpsefound = `kpsewhich --progname=$prog $f`);
+    if (lc("$d/$f") ne lc($kpsefound)) {
+      $mm->{$f} = $kpsefound;
+    }
+  }
+
+  my %mismatch;
   my $d;
   $d = "$dvipsoutputdir";
   print_and_log("  $d:\n");
   foreach my $f ('builtin35.map', 'download35.map', 'psfonts_pk.map',
                  'psfonts_t1.map', 'ps2pk.map', 'psfonts.map') {
     dir ($d, $f, 'psfonts.map');
-    $updLSR->{add}("$d/$f") unless $opts{'dry-run'};
+    if (!$opts{'dry-run'}) {
+      $updLSR->{add}("$d/$f");
+      $updLSR->{exec}();
+      $updLSR->{reset}();
+      check_mismatch(\%mismatch, $d, $f, "dvips");
+    }
   }
   $d = "$pdftexoutputdir";
   print_and_log("  $d:\n");
   foreach my $f ('pdftex_dl14.map', 'pdftex_ndl14.map', 'pdftex.map') {
     dir ($d, $f, 'pdftex.map');
-    $updLSR->{add}("$d/$f") unless $opts{'dry-run'};
+    if (!$opts{'dry-run'}) {
+      $updLSR->{add}("$d/$f");
+      $updLSR->{exec}();
+      $updLSR->{reset}();
+      check_mismatch(\%mismatch, $d, $f, "pdftex");
+    }
   }
   $d="$dvipdfmxoutputdir";
   print_and_log("  $d:\n");
   foreach my $f ('kanjix.map') {
     dir ($d, $f, '');
-    $updLSR->{add}("$d/$f") unless $opts{'dry-run'};
+    if (!$opts{'dry-run'}) {
+      $updLSR->{add}("$d/$f");
+      $updLSR->{exec}();
+      $updLSR->{reset}();
+      check_mismatch(\%mismatch, $d, $f, "dvipdfmx");
+    }
   }
   if ($pxdviUse eq "true") {
     $d="$pxdvioutputdir";
@@ -1226,8 +1279,59 @@ sub mkMaps {
     foreach my $f ('xdvi-ptex.map') {
       dir ($d, $f, '');
       $updLSR->{add}("$d/$f") unless $opts{'dry-run'};
+      if (!$opts{'dry-run'}) {
+        $updLSR->{add}("$d/$f");
+        $updLSR->{exec}();
+        $updLSR->{reset}();
+        check_mismatch(\%mismatch, $d, $f, "xdvi");
+      }
     }
   }
+
+  # all kind of warning messages
+  if ($first_time_creation_in_usermode) {
+    print_and_log("
+WARNING: you are switching to updmap's per-user mappings.
+
+You have run updmap (as opposed to updmap-sys) for the first time; this
+has created configuration files which are local to your personal account.
+
+Any changes in system map files will *not* be automatically reflected in
+your files; furthermore, running updmap-sys will no longer have any
+effect for you.  As a consequence, you have to rerun updmap yourself
+after any change in the system directories; for example, if a new font
+package is added.
+
+If you want to undo this, remove the files mentioned above.
+
+(Run $prg --help for full documentation of updmap.)
+");
+  }
+
+  if (keys %mismatch) {
+    print_and_log("
+WARNING: $prg has found mismatched files!
+
+The following files have been generated as listed above,
+but will not be found because overriding files exist, listed below.
+");
+    #
+    if ($prg eq "updmap-sys") {
+      print_and_log ("
+Perhaps you have run updmap in the past, but are running updmap-sys
+now.  Once you run updmap the first time, you have to keep using it,
+or else remove the personal configuration files it creates (the ones
+listed below).
+");
+    }
+    #
+    for my $f (sort keys %mismatch) {
+      print_and_log (" $f: $mismatch{$f}\n");
+    }
+    #
+    print_and_log("(Run $prg --help for full documentation of updmap.)\n");
+  }
+
   close LOG unless $opts{'dry-run'};
   print "\nTranscript written on \"$logfile\".\n" if !$opts{'quiet'};
 
@@ -1292,7 +1396,7 @@ sub setupOutputDir {
     if ($opts{'outputdir'}) {
       $opts{$driver . "outputdir"} = $opts{'outputdir'};
     } else {
-      $opts{$driver . "outputdir"} = "$TEXMFVAR/fonts/map/$driver/updmap";
+      $opts{$driver . "outputdir"} = "$texmfvar/fonts/map/$driver/updmap";
     }
   }
   my $od = $opts{$driver . "outputdir"};
@@ -1395,7 +1499,7 @@ sub disable_map {
   } else {
     # disable a Map type that might be activated in a lower ranked updmap.cfg
     if (!defined($alldata->{'maps'}{$map})) {
-      warning("$prg: map file not present, nothing to disable: $map\n");
+      print_warning("map file not present, nothing to disable: $map\n");
       return;
     }
     my $orig = $alldata->{'maps'}{$map}{'origin'};
@@ -1762,7 +1866,7 @@ sub merge_settings_replace_kanji {
             $alldata->{'updmap'}{$l}{'maps'}{$m}{'line'};
           $alldata->{'updmap'}{$l}{'maps'}{$newm}{'original'} = $m;
         } else {
-          print "$prg: generated map $newm (from $m) does not exists, not activating it!\n";
+          print_warning("generated map $newm (from $m) does not exists, not activating it!\n");
         }
         # in any case delete the @kanji...@ entry line, such a map will
         # never exist
@@ -1787,7 +1891,6 @@ sub merge_settings_replace_kanji {
 
 sub read_updmap_file {
   my $fn = shift;
-  my $is_old_local = ($fn =~ m/updmap-local.cfg/ ? 1 : 0);
   my %data;
   if (!open(FN,"<$fn")) {
     die ("Cannot read $fn: $!");
@@ -1801,15 +1904,6 @@ sub read_updmap_file {
   for (@lines) {
     $i++;
     chomp;
-    if ($is_old_local) {
-      # in case we read an old updmap-local.cfg we have to make sure
-      # that the disable lines
-      #       ^#!foo.map
-      # are rewritten to proper disable lines
-      #       ^#! Map foo.map
-      # we are guessing here the type of the map (namely Map)
-      $_ =~ s/^#!([^ ])/#! Map $1/;
-    }
     next if /^\s*$/;
     next if /^\s*#$/;
     next if /^\s*#[^!]/;
@@ -1824,10 +1918,10 @@ sub read_updmap_file {
       if ($b eq "Map" || $b eq "MixedMap" || $b eq "KanjiMap") {
         my $c = shift @rest;
         if (!defined($c)) {
-          warning("$prg: apparently not a real disable line, ignored: $_\n");
+          print_warning("apparently not a real disable line, ignored: $_\n");
         } else {
           if (defined($data{'maps'}{$c})) {
-            warning("$prg: double mention of $c in $fn\n");
+            print_warning("double mention of $c in $fn\n");
           }
           $data{'maps'}{$c}{'status'} = 'disabled';
           $data{'maps'}{$c}{'type'} = $b;
@@ -1837,25 +1931,25 @@ sub read_updmap_file {
       next;
     }
     if (@rest) {
-      warning("$prg: line $i in $fn contains a syntax error, more than two words!\n");
+      print_warning("line $i in $fn contains a syntax error, more than two words!\n");
     }
     if (defined($settings{$a})) {
       if (check_option($a, $b)) {
         $data{'setting'}{$a}{'val'} = $b;
         $data{'setting'}{$a}{'line'} = $i;
       } else {
-        warning("$prg: unknown setting for $a: $b, ignored!\n");
+        print_warning("unknown setting for $a: $b, ignored!\n");
       }
     } elsif ($a eq "Map" || $a eq "MixedMap" || $a eq "KanjiMap") {
       if (defined($data{'maps'}{$b}) && $data{'maps'}{$b}{'type'} ne $a) {
-        warning("$prg: double mention of $b with conflicting types in $fn\n");
+        print_warning("double mention of $b with conflicting types in $fn\n");
       } else {
         $data{'maps'}{$b}{'type'} = $a;
         $data{'maps'}{$b}{'status'} = 'enabled';
         $data{'maps'}{$b}{'line'} = $i;
       }
     } else {
-      warning("$prg: unrecognized line $i in $fn: $_\n");
+      print_warning("unrecognized line $i in $fn: $_\n");
     }
   }
   return \%data;
@@ -1939,11 +2033,11 @@ sub read_map_files {
               } else {
                 $maporig = "from " . $alldata->{'maps'}{$fontorig}{'origin'};
               }
-              warning("$prg: font $font is defined multiple times:\n");
-              warning("$prg:   $fontorig ($maporig)\n");
-              warning("$prg:   $m (from $f) (used)\n");
+              print_warning("font $font is defined multiple times:\n");
+              print_warning("  $fontorig ($maporig)\n");
+              print_warning("  $m (from $f) (used)\n");
             } else {
-              warning("$prg: font $font is multiply defined in $m, using an arbitrary instance!\n");
+              print_warning("font $font is multiply defined in $m, using an arbitrary instance!\n");
             }
           }
           $alldata->{'fonts'}{$font}{'origin'} = $m;
@@ -2031,29 +2125,31 @@ sub reset_root_home {
       my (undef,undef,undef,undef,undef,undef,undef,$roothome) = getpwuid(0);
       if (defined($roothome)) {
         if ($envhome ne $roothome) {
-          warning("$prg: resetting \$HOME value (was $envhome) to root's "
+          print_warning("resetting \$HOME value (was $envhome) to root's "
             . "actual home ($roothome).\n");
           $ENV{'HOME'} = $roothome;
         } else {
           # envhome and roothome do agree, nothing to do, that is the good case
         }
       } else { 
-        warning("$prg: home of root not defined, strange!\n");
+        print_warning("home of root not defined, strange!\n");
       }
     }
   }
 }
 
-sub warning {
-  print STDERR @_;
+sub print_warning {
+  print STDERR "$prg [WARNING]: ", @_ if (!$opts{'quiet'}) 
+}
+sub print_error {
+  print STDERR "$prg [ERROR]: ", @_;
 }
 
-#
-# help, version etc etc
-#
+
+# help, version.
 
 sub version {
-  my $ret = sprintf "%s (TeX Live, multi) version %s\n", $prg, $version;
+  my $ret = sprintf "%s version %s\n", $prg, $version;
   return $ret;
 }
 
@@ -2062,16 +2158,37 @@ sub help {
 Usage: $prg     [OPTION] ... [COMMAND]
    or: $prg-sys [OPTION] ... [COMMAND]
 
-Update the default font map files used by pdftex, dvips, and dvipdfm(x),
-and optionally pxdvi, as determined by all configuration files updmap.cfg 
-(the ones returned by running "kpsewhich --all updmap.cfg", but see below).
+Update the default font map files used by pdftex (pdftex.map), dvips
+(psfonts.map), and dvipdfm(x), and optionally pxdvi, as determined by
+all configuration files updmap.cfg (the ones returned by running
+"kpsewhich --all updmap.cfg", but see below).
 
 Among other things, these map files are used to determine which fonts
 should be used as bitmaps and which as outlines, and to determine which
-font files are included in the PDF or PostScript output.
+font files are included, typically subsetted, in the PDF or PostScript output.
+
+updmap-sys is intended to affect the system-wide configuration, while
+updmap affects personal configuration files only, overriding the system
+files.  As a consequence, once updmap has been run, even a single time,
+running updmap-sys no longer has any effect.  (updmap-sys issues a
+warning in this situation.)
 
 By default, the TeX filename database (ls-R) is also updated.
 
+The updmap system is regrettably complicated, for both inherent and
+historical reasons.  A general overview:
+- updmap.cfg files are mainly about listing other files, namely the
+  font-specific .maps, in which each line gives information about a
+  different TeX (.tfm) font.
+- updmap reads the updmap.cfg files and then concatenates the
+  contents of those .map files into the main output files: psfonts.map
+  for dvips and pdftex.map for pdftex and dvipdfmx.
+- The updmap.cfg files themselves are created and updated at package
+  installation time, by the system installer or the package manager or
+  by hand, and not (by default) by updmap.
+
+Good luck.
+
 Options:
   --cnffile FILE            read FILE for the updmap configuration 
                              (can be given multiple times, in which case
@@ -2085,6 +2202,7 @@ Options:
   --force                   recreate files even if config hasn't changed
   --nomkmap                 do not recreate map files
   --nohash                  do not run texhash
+  --sys                     affect system-wide files (equivalent to updmap-sys)
   -n, --dry-run             only show the configuration, no output
   --quiet, --silent         reduce verbosity
 
@@ -2115,19 +2233,20 @@ KanjiMap entries are added to psfonts_t1.map and kanjix.map.
 
 Explanation of the OPTION names for --showoptions, --showoption, --setoption:
 
-  dvipsPreferOutline    true|false  (default true)
+  dvipsPreferOutline    true,false  (default true)
     Whether dvips uses bitmaps or outlines, when both are available.
-  dvipsDownloadBase35   true|false  (default true)
+  dvipsDownloadBase35   true,false  (default true)
     Whether dvips includes the standard 35 PostScript fonts in its output.
-  pdftexDownloadBase14  true|false   (default true)
+  pdftexDownloadBase14  true,false   (default true)
     Whether pdftex includes the standard 14 PDF fonts in its output.
-  pxdviUse              true|false  (default false)
+  pxdviUse              true,false  (default false)
     Whether maps for pxdvi (Japanese-patched xdvi) are under updmap's control.
   kanjiEmbed            (any string)
   kanjiVariant          (any string)
     See below.
-  LW35                  URWkb|URW|ADOBEkb|ADOBE  (default URWkb)
+  LW35                  URWkb,URW,ADOBEkb,ADOBE  (default URWkb)
     Adapt the font and file names of the standard 35 PostScript fonts.
+
     URWkb    URW fonts with "berry" filenames    (e.g. uhvbo8ac.pfb)
     URW      URW fonts with "vendor" filenames   (e.g. n019064l.pfb)
     ADOBEkb  Adobe fonts with "berry" filenames  (e.g. phvbo8an.pfb)
@@ -2147,12 +2266,13 @@ Explanation of the OPTION names for --showoptions, --showoption, --setoption:
 
 Explanation of trees and files normally used:
 
-  If --cnffile is specified on the command line (possibly multiple
-  times), its value(s) are used.  Otherwise, updmap reads all the
-  updmap.cfg files found by running \`kpsewhich -all updmap.cfg', in the
-  order returned by kpsewhich.
+  If --cnffile is specified on the command line (can be given multiple
+  times), its value(s) is(are) used.  Otherwise, updmap reads all the
+  updmap.cfg files found by running \`kpsewhich -all updmap.cfg',
+  in the order returned by kpsewhich (which is the order of trees
+  defined in texmf.cnf).
 
-  In any case, if multiple updmap.cfg files are found, all the maps
+  In either case, if multiple updmap.cfg files are found, all the maps
   mentioned in all the updmap.cfg files are merged.
 
   Thus, if updmap.cfg files are present in all trees, and the default
@@ -2176,75 +2296,96 @@ Explanation of trees and files normally used:
   
   (where YYYY is the TeX Live release version).
   
-  There is one exception to keep upgradability from earlier versions
-  of TeX Live: if a file TEXMFLOCAL/web2c/updmap-local.cfg exists
-  (formerly used by tlmgr to merge local fonts), then the file
-  TEXMFLOCAL/web2c/updmap.cfg is ignored (if it exists) and that
-  updmap-local.cfg is used instead.  In this case, updmap recognizes the
-  previous syntax for disabling map files in updmap-local.cfg (this
-  syntax is different from what is used now).
-
   According to the actions, updmap might write to one of the given files
   or create a new updmap.cfg, described further below.
 
-  Where changes are saved: if config files are given on the command
-  line, then the first one given will be used to save any changes from
-  --setoption, --enable or --disable.  If the config files are taken
-  from kpsewhich output, then the algorithm is more complex:
+Where and which updmap.cfg changes are saved: 
 
-    1) If \$TEXMFCONFIG/web2c/updmap.cfg or \$TEXMFHOME/web2c/updmap.cfg
+  When no options are given, the updmap.cfg file(s) are only read, not
+  written.  It's when an option --setoption, --enable or --disable is
+  specified that an updmap.cfg needs to be updated.  In this case:
+
+  1) If config files are given on the command line, then the first one
+  given will be used to save any such changes.
+  
+  2) If the config files are taken from kpsewhich output, then the
+  algorithm is more complex:
+
+    2a) If \$TEXMFCONFIG/web2c/updmap.cfg or \$TEXMFHOME/web2c/updmap.cfg
     appears in the list of used files, then the one listed first by
     kpsewhich --all (equivalently, the one returned by kpsewhich
     updmap.cfg), is used.
       
-    2) If neither of the above two are present and changes are made, a
+    2b) If neither of the above two are present and changes are made, a
     new config file is created in \$TEXMFCONFIG/web2c/updmap.cfg.
+  
+  In general, the idea is that if the user cannot write to a given
+  config file, a higher-level one can be used.  That way, the
+  distribution's settings can be overridden system-wide using
+  TEXMFLOCAL, and system settings can be overridden again in a
+  particular user's TEXMFHOME.
 
-  Resolving multiple definitions of a font:
-    If a font is defined in more than one map file, then the definition
-    coming from the first-listed updmap.cfg is used.  If a font is
-    defined multiple times within the same map file, one is chosen
-    arbitrarily.  In both cases a warning is issued.
+Resolving multiple definitions of a font:
 
-  Disabling maps:
-    updmap.cfg files with higher priority (listed earlier) can disable
-    maps mentioned in lower priority (listed later) updmap.cfg files by
-    writing, e.g.,
-      \#! Map mapname.map
-    or
-      \#! MixedMap mapname.map
-    in the higher-priority updmap.cfg file. 
+  If a font is defined in more than one map file, then the definition
+  coming from the first-listed updmap.cfg is used.  If a font is
+  defined multiple times within the same map file, one is chosen
+  arbitrarily.  In both cases a warning is issued.
 
-    As an example, suppose you have a copy of MathTime Pro fonts
-    and want to disable the Belleek version of the fonts; that is,
-    disable the map belleek.map.  You can create the file
-    \$TEXMFCONFIG/web2c/updmap.cfg with the content
-      #! Map belleek.map
-      Map mt-plus.map
-      Map mt-yy.map
-    and call $prg.
+Disabling maps:
 
-  updmap writes the map files for dvips (psfonts.map) and pdftex
-  (pdftex.map) to the TEXMFVAR/fonts/map/updmap/{dvips,pdftex}/
-  directories.   
+  updmap.cfg files with higher priority (listed earlier) can disable
+  maps mentioned in lower priority (listed later) updmap.cfg files by
+  writing, e.g.,
+    \#! Map mapname.map
+  or
+    \#! MixedMap mapname.map
+  in the higher-priority updmap.cfg file.  (The \#! must be at the
+  beginning of the line, with at least one space or tab afterward, and
+  whitespace between each word on the list.)
 
-  The log file is written to TEXMFVAR/web2c/updmap.log.
+  As an example, suppose you have a copy of MathTime Pro fonts
+  and want to disable the Belleek version of the fonts; that is,
+  disable the map belleek.map.  You can create the file
+  \$TEXMFCONFIG/web2c/updmap.cfg with the content
+    #! Map belleek.map
+    Map mt-plus.map
+    Map mt-yy.map
+  and call $prg.
+
+The main output:
+
+  The main output of updmap is the files containing the individual font
+  map lines which the drivers (dvips, pdftex, etc.) read to handle fonts.
+  
+  The map files for dvips (psfonts.map) and pdftex (pdftex.map) are
+  written to TEXMFVAR/fonts/map/updmap/{dvips,pdftex}/.
+  
+  In addition, information about Kanji fonts is written to
+  TEXMFVAR/fonts/map/updmap/dvipdfmx/kanjix.map, and optionally to 
+  TEXMFVAR/fonts/map/updmap/pxdvi/xdvi-ptex.map.  These are for Kanji
+  only and are not like other map files.  dvipdfmx reads pdftex.map for
+  the map entries for non-Kanji fonts.
+
+updmap vs. updmap-sys:
 
   When updmap-sys is run, TEXMFSYSCONFIG and TEXMFSYSVAR are used
-  instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the only
-  difference between updmap-sys and updmap.
+  instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the
+  primary difference between updmap-sys and updmap.
 
   Other locations may be used if you give them on the command line, or
   these trees don't exist, or you are not using the original TeX Live.
 
-  To see the precise locations of the various files that
-  will be read and written, give the -n option (or read the source).
+To see the precise locations of the various files that
+will be read and written, give the -n option (or read the source).
+
+The log file is written to TEXMFVAR/web2c/updmap.log.
 
 For step-by-step instructions on making new fonts known to TeX, read
 http://tug.org/fonts/fontinstall.html.  For even more terse
-instructions, read the beginning of updmap.cfg.
+instructions, read the beginning of the main updmap.cfg.
 
-Report bugs to: tex-k\@tug.org
+Report bugs to: tex-live\@tug.org
 TeX Live home page: <http://tug.org/texlive/>
 EOF
 ;
